Option Strict On
Option Explicit On
Imports System.Text
Imports SecureObjectCore.Security.Object

Public Module SecureObjectTest
   Private Rand As New Random
   Public Sub Main(args As String())
      SampleStringBytes()
      SamlesClass()
      SamplesStaticShared()
   End Sub

   Private Sub SampleStringBytes()
      Dim mult = 5I

      Dim text = "My SecureText, My SecureText, My SecureText "
      Dim sb = New StringBuilder
      Enumerable.Range(0, mult).ToList.ForEach(Sub(x) sb.Append(text))

      Dim plain = sb.ToString
      Dim plainsize = plain.Length
      Dim bplain = Encoding.UTF8.GetBytes(plain)

      Using secureobj = New SecureObject(Of String)
         '**** **** **** **** **** **** **** **** **** 
         'WICHTIG: String sind nicht gerne gesehen, in
         '         Arbeiten im Securebereich !!!!!!!!
         ' >> Besser statt String GENERELL mit Bytes 
         '    arbeiten.
         '**** **** **** **** **** **** **** **** **** 

         'Ist nicht mglich, da SecureObject(Of String)
         'und nicht SecureObject(Of Byte())
         '  >> Es wird ein Exception geworfen !
         'secureobj.InitNew(bplain)

         'Gewnschter String eingeben
         secureobj.InitNew(plain)

         'Um den String aus dem Securebereich extrahieren
         Dim decipher = secureobj.ToObject

         'Kurze Prfung ob beide "Strings" gleich sind.
         If Not secureobj.ToObject.SequenceEqual(plain) Then Stop
      End Using

      Using secureobj = New SecureObject(Of Byte())
         '**** **** **** **** **** **** **** **** **** **** **** 
         'Byte-Arrays besitzen eine generisch spezialisierte
         'Methode, mit dem Zusatzparameter '_nothingplain'
         'Die Byte-Array kann also gleich wieder zurckgesetzt
         'werden auf Nothing, sofern erwnscht.
         '**** **** **** **** **** **** **** **** **** **** **** 

         'Gewnschtes Byte-Array eingeben
         'False >> ohne Array-Rcksetzung
         secureobj.InitNew(bplain, False)

         'Die Byte-Array aus dem Securebereich extrahieren
         'Dim decipher = secureobj.ToObject

         'Kurze Prfung ob beide Byte-Arrays gleich sind.
         If Not secureobj.ToObject.SequenceEqual(bplain) Then Stop

         'Eine Kopie von SecureObject(Of T)
         Dim copyso = secureobj.Copy

         'Beide SecureObject(Of T) auf Gleichheit prfen
         If Not secureobj.Equals(copyso) Then Stop

         'WICHTIG: Nach Clear kann SecureObject(Of T) weiterhin
         '         verwendet werden.
         '         (Typengleichheit muss beachtet werden)
         secureobj.Clear()

         'Prfen ob SecureObject(Of T) schon besetzt ist
         If secureobj.IsBusy Then Stop

         'Gewnschtes Byte-Array eingeben
         secureobj.InitNew(bplain, False)
         If Not secureobj.Equals(copyso) Then Stop

         'WICHTIG: Nach dem Disposen ist es nicht mehr
         '         mglich mit SecureObject zu arbeiten
         '         >> copyso.IsDisposed = False 
         copyso.Dispose()

         'Prfen ob 'copyso' Disposed
         If Not copyso.IsDisposed Then Stop

         'Da 'copyso' Disposed wird ein False zurckgegeben
         If secureobj.Equals(copyso) Then Stop

         'Idealerweise immer auf Nothing setzen
         copyso = Nothing
      End Using
   End Sub
   Private Sub SamlesClass()
      Dim samepersone() As Person
      Dim persons = New Person() {
         New Person With {.FirstName = "Al", .LastName = "Pacino", .Occupation = "Actor", .Born = New DateTime(1940, 4, 25), .Age = Person.CalcAge(.Born)},
         New Person With {.FirstName = "Jeanne Louise", .LastName = "Calment", .Occupation = "Supercentenarian", .Born = New DateTime(1875, 2, 21), .Age = Person.CalcAge(.Born)},
         New Person With {.FirstName = "Pablo", .LastName = "Picasso", .Occupation = "Painter", .Born = New DateTime(1881, 10, 25), .Age = Person.CalcAge(.Born)},
         New Person With {.FirstName = "Abraham", .LastName = "Lincoln", .Occupation = "Prsident", .Born = New DateTime(1809, 2, 12), .Age = Person.CalcAge(.Born)},
         New Person With {.FirstName = "Louis", .LastName = "Armstrong", .Occupation = "Musician", .Born = New DateTime(1901, 8, 4), .Age = Person.CalcAge(.Born)},
         New Person With {.FirstName = "Carl Friedrich", .LastName = "Gau", .Occupation = "Mathematician", .Born = New DateTime(1777, 4, 30), .Age = Person.CalcAge(.Born)}
      }

      Using secureobj = New SecureObject(Of Person())
         'Gewnschtes Object eingeben
         secureobj.InitNew(persons)

         'Eine Kopie von SecureObject(Of T)
         Dim copyso = secureobj.Copy

         'Beide SecureObject(Of T) auf Gleichheit prfen
         'WICHTIG: Gleichheit heisst auch gleiche Reihenfolge
         '         Durch die Komplexheit von Objekten leider 
         '         so vorgegeben
         If Not secureobj.Equals(copyso) Then Stop

         'WICHTIG: Nach Clear kann SecureObject(Of T) weiterhin
         '         verwendet werden.
         '         (Typengleichheit muss beachtet werden)
         copyso.Clear()

         'Mischen von Person
         Shuffel(persons)

         'Gewnschtes Object eingeben
         copyso.InitNew(persons)

         'Beide SecureObject(Of T) auf Gleichheit prfen
         'WICHTIG: Gleichheit heisst auch gleiche Reihenfolge
         '         >> Durch das Mischen nicht mehr gleich
         If secureobj.Equals(copyso) Then Stop

         'Nochmals Clear
         copyso.Clear()

         'Ein X-beliebiges Element von 'persons' entfernen
         persons = persons.Where(Function(p) Not p.FirstName = "Al").ToArray

         'Gewnschtes Object eingeben
         copyso.InitNew(persons)

         'Beide SecureObject(Of T) auf Gleichheit prfen
         If secureobj.Equals(copyso) Then Stop

         'Das Objekt extrahieren
         samepersone = secureobj.ToObject
         Person.SetAge(samepersone)
      End Using
   End Sub
   Private Sub SamplesStaticShared()
      Dim size As Int32 = 25

      'Eine Anzahl Random-Char (UniCode) in einer Char-Array
      Dim rngchar = SecureObject.GetRngChar(size)

      'Eine Anzahl Random-Char vom Alphabet in einer Char-Array
      Dim rngalphachar = SecureObject.GetRndAlpha(size)

      'Eine Anzahl Random-Char (UniCode) in einer Byte-Array UTF8
      Dim rngcharbytes = SecureObject.GetRngCharBytesU8(size)

      'Eine Anzahl Random-Char (UniCode) in einer Byte-Array UTF8
      'wobei sich 'size' auf die Byteanzahl bezieht
      Dim rngcharbytesfix = SecureObject.GetRngCharBytesU8Fix(size)

      'Binary-Serialisierung
      Dim seribin = SecureObject.SerializeBinary(rngcharbytes)

      'Binary-Deserialisierung
      Dim deseribin = SecureObject.DeserializeBinary(Of Byte())(seribin)

      'Eine Anzahl Random-Bytes (ohne 0)
      Dim rngbytes = SecureObject.RandomKey(size)
   End Sub
   Private Sub Shuffel(ByRef persons() As Person)
      persons = persons.OrderBy(Function(p) Rand.Next).ToArray
   End Sub
End Module

<Serializable>
Public Class Person
   Public FirstName As String
   Public LastName As String
   Public Occupation As String
   Public Born As DateTime

   <NonSerialized>
   Public Age As Int32

   Public Shared Function CalcAge(birthdate As DateTime) As Int32
      Dim today = DateTime.Today
      Dim age = today.Year - birthdate.Year
      Return If(birthdate.Date > today.AddYears(-age), age - 1, age)
   End Function
   Public Shared Sub SetAge(ByRef _person As Person)
      If _person Is Nothing Then Return
      _person.Age = CalcAge(_person.Born)
   End Sub
   Public Shared Sub SetAge(ByRef persons() As Person)
      If persons Is Nothing Then Return
      If persons.Length < 1 Then Return
      For i As Int32 = 0 To persons.Length - 1
         SetAge(persons(i))
      Next
   End Sub
End Class