﻿Option Strict On
Option Explicit On
Imports System.Text
Imports SecureObjectFW.Security.Object

Public Module SecureObjectTest
   Private Rand As New Random
   Public Sub Main()
      SampleStringBytes()
      SamlesClass()
      SamplesStaticShared()
      SamplesDataset()

      Console.WriteLine()
      Console.WriteLine("FINISH")
      Console.ReadLine()
   End Sub

   Private Sub SampleStringBytes()
      Dim mult = 100I

      Dim text = "My SecureText, My SecureText, My SecureText "
      Dim sb = New StringBuilder
      Enumerable.Range(0, mult).ToList.ForEach(Sub(x) sb.Append(text))

      Dim plain = sb.ToString
      Dim plainsize = plain.Length
      Dim bplain = Encoding.UTF8.GetBytes(plain)

      Console.WriteLine("SecureObject(Of String), size = {0}", plain.Length)
      Using secureobj = New SecureObject(Of String)
         '**** **** **** **** **** **** **** **** **** 
         'WICHTIG: String wird nicht gerne gesehen, in
         '         Arbeiten im Securebereich !!!!!!!!
         ' >> Besser statt String GENERELL mit Bytes 
         '    arbeiten.
         '**** **** **** **** **** **** **** **** **** 

         'Ist nicht möglich, da SecureObject(Of String)
         'und nicht SecureObject(Of Byte())
         '  >> Es wird ein Exception geworfen !
         'secureobj.InitNew(bplain)

         'Gewünschter String eingeben
         secureobj.InitNew(plain)

         'Um den String aus dem Securebereich extrahieren
         Dim decipher = secureobj.ToObject

         'Kurze Prüfung ob beide "Strings" gleich sind.
         If Not secureobj.ToObject.SequenceEqual(plain) Then Stop
         Console.WriteLine("Equals", decipher.SequenceEqual(plain))
      End Using

      Console.WriteLine()
      Console.WriteLine("SecureObject(Of Byte()), size = {0}", bplain.Length)
      Using secureobj = New SecureObject(Of Byte())
         '**** **** **** **** **** **** **** **** **** **** **** 
         'Byte-Arrays besitzen eine generisch spezialisierte
         'Methode, mit dem Zusatzparameter '_nothingplain'
         'Die Byte-Array kan also gleich wieder zurückgesetzt
         'werden auf Nothing, sofern erwünscht.
         '**** **** **** **** **** **** **** **** **** **** **** 

         'Gewünschtes Byte-Array eingeben
         'False >> ohne Array-Rücksetzung
         secureobj.InitNew(bplain, False)

         'Die Byte-Array aus dem Securebereich extrahieren
         Dim decipher = secureobj.ToObject

         'Kurze Prüfung ob beide Byte-Arrays gleich sind.
         If Not secureobj.ToObject.SequenceEqual(bplain) Then Stop
         Console.WriteLine("Equals", decipher.SequenceEqual(bplain))

         'Eine Kopie von SecureObject(Of T)
         Dim copyso = secureobj.Copy

         'Beide SecureObject(Of T) auf Gleichheit prüfen
         If Not secureobj.Equals(copyso) Then Stop

         'WICHTIG: Nach Clear kann SecureObject(Of T) weiterhin
         '         verwendet werden.
         '         (Typengleichheit muss beachtet werden)
         secureobj.Clear()

         'Prüfen ob SecureObject(Of T) schon besetzt ist
         If secureobj.IsBusy Then Stop

         'Gewünschtes Byte-Array eingeben
         secureobj.InitNew(bplain, False)
         If Not secureobj.Equals(copyso) Then Stop

         'WICHTIG: Nach dem Disposen ist es nicht mehr
         '         möglich mit SecureObject zu arbeiten
         '         >> copyso.IsDisposed = False 
         copyso.Dispose()

         'Prüfen ob 'copyso' Disposed
         If Not copyso.IsDisposed Then Stop

         'Da 'copyso' Disposed wird ein False zurückgegeben
         If secureobj.Equals(copyso) Then Stop

         'Idealerweise immer auf Nothing setzen
         copyso = Nothing
      End Using
   End Sub
   Private Sub SamlesClass()
      Dim samepersone() As Person
      Dim persons = New Person() {
         New Person With {.FirstName = "Al", .LastName = "Pacino", .Occupation = "Actor", .Born = New DateTime(1940, 4, 25), .Age = Person.CalcAge(.Born)},
         New Person With {.FirstName = "Jeanne Louise", .LastName = "Calment", .Occupation = "Supercentenarian", .Born = New DateTime(1875, 2, 21), .Age = Person.CalcAge(.Born)},
         New Person With {.FirstName = "Pablo", .LastName = "Picasso", .Occupation = "Painter", .Born = New DateTime(1881, 10, 25), .Age = Person.CalcAge(.Born)},
         New Person With {.FirstName = "Abraham", .LastName = "Lincoln", .Occupation = "Präsident", .Born = New DateTime(1809, 2, 12), .Age = Person.CalcAge(.Born)},
         New Person With {.FirstName = "Louis", .LastName = "Armstrong", .Occupation = "Musician", .Born = New DateTime(1901, 8, 4), .Age = Person.CalcAge(.Born)},
         New Person With {.FirstName = "Carl Friedrich", .LastName = "Gauß", .Occupation = "Mathematician", .Born = New DateTime(1777, 4, 30), .Age = Person.CalcAge(.Born)}
      }

      Console.WriteLine()
      Console.WriteLine("SecureObject(Of Person()), size = {0}", persons.Length)
      Using secureobj = New SecureObject(Of Person())
         'Gewünschtes Object eingeben
         secureobj.InitNew(persons)

         'Eine Kopie von SecureObject(Of T)
         Dim copyso = secureobj.Copy

         'Beide SecureObject(Of T) auf Gleichheit prüfen
         'WICHTIG: Gleichheit heisst auch gleiche Reihenfolge
         '         Durch die Komplexheit von Objekten leider 
         '         so vorgegeben
         If Not secureobj.Equals(copyso) Then Stop
         Console.WriteLine("Equals", secureobj.Equals(copyso))

         'WICHTIG: Nach Clear kann SecureObject(Of T) weiterhin
         '         verwendet werden.
         '         (Typengleichheit muss beachtet werden)
         copyso.Clear()

         'Mischen von Person
         Shuffel(persons)

         'Gewünschtes Object eingeben
         copyso.InitNew(persons)

         'Beide SecureObject(Of T) auf Gleichheit prüfen
         'WICHTIG: Gleichheit heisst auch gleiche Reihenfolge
         '         >> Durch das Mischen nicht mehr gleich
         If secureobj.Equals(copyso) Then Stop

         'Nochmals Clear
         copyso.Clear()

         'Ein X-beliebiges Element von 'persons' entfernen
         persons = persons.Where(Function(p) Not p.FirstName = "Al").ToArray

         'Gewünschtes Object eingeben
         copyso.InitNew(persons)

         'Beide SecureObject(Of T) auf Gleichheit prüfen
         If secureobj.Equals(copyso) Then Stop

         'Das Objekt extrahieren
         samepersone = secureobj.ToObject
         Person.SetAge(samepersone)
      End Using
   End Sub
   Private Sub SamplesDataset()
      'DataSet/DataTable sind sehr komplexe Gebilde, 
      'hinter dem beim Designen, sogar Code generiert 
      'wird. 
      'Einfach das Dataset/DataTable in das SecureObject(Of T)
      'einzureichen wird fehlschlagen. Gute Erfahrungen habe 
      'ich mit vorgängigem Serialisieren und Deserialisieren 
      'gemacht. Ob das jedoch immer funktioniert müsste aus-
      'führlich getestet und analysiert werden.

      Dim persons = New Person() {
        New Person With {.FirstName = "Al", .LastName = "Pacino", .Occupation = "Actor", .Born = New DateTime(1940, 4, 25), .Age = Person.CalcAge(.Born)},
        New Person With {.FirstName = "Jeanne Louise", .LastName = "Calment", .Occupation = "Supercentenarian", .Born = New DateTime(1875, 2, 21), .Age = Person.CalcAge(.Born)},
        New Person With {.FirstName = "Pablo", .LastName = "Picasso", .Occupation = "Painter", .Born = New DateTime(1881, 10, 25), .Age = Person.CalcAge(.Born)},
        New Person With {.FirstName = "Abraham", .LastName = "Lincoln", .Occupation = "Präsident", .Born = New DateTime(1809, 2, 12), .Age = Person.CalcAge(.Born)},
        New Person With {.FirstName = "Louis", .LastName = "Armstrong", .Occupation = "Musician", .Born = New DateTime(1901, 8, 4), .Age = Person.CalcAge(.Born)},
        New Person With {.FirstName = "Carl Friedrich", .LastName = "Gauß", .Occupation = "Mathematician", .Born = New DateTime(1777, 4, 30), .Age = Person.CalcAge(.Born)}
      }
      Dim dspersons = New DsPerson
      For Each person As Person In persons
         With person
            dspersons.Person.AddPersonRow(.FirstName, .LastName, .Occupation, .Born)
         End With
      Next

      'Damit die Equalsprüfung nachher stimmt, muss
      'das Originaldataset von irgendwelchen Items
      'die nicht seiralisierbar sind bereinigt werden.
      Dim dspersonsserislize = SecureObject.SerializeBinary(dspersons)
      Dim dspersonsnew = SecureObject.DeserializeBinary(Of DsPerson)(dspersonsserislize)
      Dim firstrow = DirectCast(dspersonsnew.Tables(0), DsPerson.PersonDataTable).First.ItemArray

      Using secureobj = New SecureObject(Of DsPerson)
         'Gewünschtes Dataset einreichen
         secureobj.InitNew(dspersonsnew)

         'DsPerson aus dem Securebereich extrahieren
         Dim decipher = secureobj.ToObject

         Dim copyso = secureobj.Copy
         If Not secureobj.Equals(copyso) Then Stop

         copyso.Dispose()
         copyso = Nothing
      End Using

      'Genau das gleiche Spiel auch bei den Datatables
      Dim dtperson = DirectCast(dspersons.Tables(0), DsPerson.PersonDataTable)
      Dim dtpersonserialize = SecureObject.SerializeBinary(dtperson)
      Dim dtpersonsnew = SecureObject.DeserializeBinary(Of DsPerson.PersonDataTable)(dtpersonserialize)

      firstrow = dtpersonsnew.First.ItemArray
      Using secureobj = New SecureObject(Of DsPerson.PersonDataTable)
         secureobj.InitNew(dtpersonsnew)

         'DtPerson aus dem Securebereich extrahieren
         Dim decipher = secureobj.ToObject

         Dim copyso = secureobj.Copy
         If Not secureobj.Equals(copyso) Then Stop

         copyso.Dispose()
         copyso = Nothing
      End Using
   End Sub
   Private Sub SamplesStaticShared()
      Dim size As Int32 = 25

      'Eine Anzahl Random-Char (UniCode) in einer Char-Array
      Dim rngchar = SecureObject.GetRngChar(size)

      'Eine Anzahl Random-Char vom Alphabet in einer Char-Array
      Dim rngalphachar = SecureObject.GetRndAlpha(size)

      'Eine Anzahl Random-Char (UniCode) in einer Byte-Array UTF8
      Dim rngcharbytes = SecureObject.GetRngCharBytesU8(size)

      'Eine Anzahl Random-Char (UniCode) in einer Byte-Array UTF8
      'wobei sich 'size' auf die Byteanzahl bezieht
      Dim rngcharbytesfix = SecureObject.GetRngCharBytesU8Fix(size)

      'Binary-Serialisierung
      Dim seribin = SecureObject.SerializeBinary(rngcharbytes)

      'Binary-Deserialisierung
      Dim deseribin = SecureObject.DeserializeBinary(Of Byte())(seribin)

      'Eine Anzahl Random-Bytes (ohne 0)
      Dim rngbytes = SecureObject.RandomKey(size)
   End Sub
   Private Sub Shuffel(ByRef persons() As Person)
      persons = persons.OrderBy(Function(p) Rand.Next).ToArray
   End Sub
End Module

<Serializable>
Public Class Person
   Public FirstName As String
   Public LastName As String
   Public Occupation As String
   Public Born As DateTime

   <NonSerialized>
   Public Age As Int32

   Public Shared Function CalcAge(birthdate As DateTime) As Int32
      Dim today = DateTime.Today
      Dim age = today.Year - birthdate.Year
      Return If(birthdate.Date > today.AddYears(-age), age - 1, age)
   End Function
   Public Shared Sub SetAge(ByRef _person As Person)
      If _person Is Nothing Then Return
      _person.Age = CalcAge(_person.Born)
   End Sub
   Public Shared Sub SetAge(ByRef persons() As Person)
      If persons Is Nothing Then Return
      If persons.Length < 1 Then Return
      For i As Int32 = 0 To persons.Length - 1
         SetAge(persons(i))
      Next
   End Sub
End Class