﻿Option Strict On
Option Explicit On
Imports System.IO
Imports System.Text
Imports System.ComponentModel
Imports System.Security.Cryptography
Imports System.Runtime.InteropServices
Imports System.Runtime.Serialization.Formatters.Binary
Imports RSO = SecureObjectCore.Security.Object.MyRngSecureObject.SOCR

'Für Threads nicht getestet.

'Alle Variablen die deklariert und instanziert 
'werden, werden nach erfolgter Tätigkeit gleich 
'wieder zurückgesetzt. 

'SecureObject besteht aus 3 Klassen
'- Klasse SecureObject(Of T)
'- Klasse SecureObject für Static/Shared
'- Klasse SOCR (Einfache Int32-Random)

Namespace Security.Object
   ''' <summary>
   ''' Represents a Object that should be kept confidential, 
   ''' such as by deleting it from computer memory when no longer needed. 
   ''' </summary>
   ''' <typeparam name="T">the desired datatype</typeparam>
   <DebuggerNonUserCode>
   Public NotInheritable Class SecureObject(Of T)
      Implements IDisposable
#Region "Members and Variables"
      Public IsBusy As Boolean
      Public IsDisposed As Boolean
      ''' <summary>
      ''' Invisible variables
      ''' </summary>
      <Browsable(False)>
      <DebuggerBrowsable(DebuggerBrowsableState.Never)>
      <DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)>
      Private Seed As IntPtr, Key As IntPtr, KeyLen As IntPtr
      ''' <summary>
      ''' Invisible variables
      ''' </summary>
      <Browsable(False)>
      <DebuggerBrowsable(DebuggerBrowsableState.Never)>
      <DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)>
      Private LengthInternal As Int32, SecurePtr As IntPtr, Gch As List(Of GCHandle)
#End Region
#Region "Entries"
      ''' <summary>
      ''' Make a copy of ME as SecureObject(Of T)
      ''' </summary>
      ''' <returns>Return a copy of ME as SecureObject(Of T)</returns>
      Public Function Copy() As SecureObject(Of T)
         If Not Me.IsBusy Then Return Nothing
         If Me.IsDisposed Then Return Nothing
         Dim res = New SecureObject(Of T)
         res.InitNew(Me.ToObject)
         Return res
      End Function
      ''' <summary>
      ''' Check if two SecureObject are equal
      ''' </summary>
      ''' <param name="mso">SecureObject</param>
      ''' <returns>True for Equals = true</returns>
      Public Shadows Function Equals(mso As SecureObject(Of T)) As Boolean
         If mso Is Nothing Then Return False
         If Not Me.IsBusy Then Return False
         If Me.IsDisposed Then Return False
         If Not mso.IsBusy Then Return False
         If mso.IsDisposed Then Return False

         Return SecureObject.SerializeBinary(mso.ToObject).
                SequenceEqual(Me.GetPlain)
      End Function
      ''' <summary>
      ''' Initialization MySecureObject new
      ''' </summary>
      ''' <param name="plain">A Object as plain</param>
      Public Sub InitNew(plain As T)
         If Me.IsBusy Then Return
         If Me.IsDisposed Then Return

         Me.Clear(True)
         If plain IsNot Nothing Then
            Dim plainbyte = SecureObject.SerializeBinary(plain)
            Me.InitializeSO(plainbyte, True)
         End If
      End Sub
      ''' <summary>
      ''' Initialization MySecureObject new
      ''' </summary>
      ''' <param name="plain">A Array of Byte</param>
      ''' <param name="_nothingplain">Reset plain</param>
      Public Sub InitNew(ByRef plain() As Byte,
                 Optional _nothingplain As Boolean = True)
         If Me.IsBusy Then Return
         If Me.IsDisposed Then Return
         If plain IsNot Nothing AndAlso plain.Length > 8 Then
            If Not GetType(T) = plain.GetType Then
               Dim e = New Exception("SecureObject-InitializationType is not a array of Byte")
               Throw New TypeInitializationException(GetType(T).ToString, e)
            End If
            Me.Clear(True)
            Dim plainbyte = SecureObject.SerializeBinary(plain)
            Me.InitializeSO(plainbyte, True)
            If _nothingplain Then Me.ZeroFree(plain)
            Return
         End If
         If _nothingplain Then Me.ZeroFree(plain)
      End Sub
      ''' <summary>
      ''' Get to Object
      ''' </summary>
      ''' <returns>Return the plainobject</returns>
      Public Function ToObject() As T
         If Me.IsDisposed Then Return Nothing
         Return SecureObject.DeserializeBinary(Of T)(Me.GetPlain())
      End Function
      ''' <summary>
      ''' Clears all resources to allow a new initialization
      ''' </summary>
      ''' <param name="_withInitNew">Optional with new Initialize</param>
      Public Sub Clear(Optional _withInitNew As Boolean = False)
         If Me.IsDisposed Then Return
         Me.IsBusy = False
         Me.IsDisposed = False
         Me.WarmUp()
         Me.Seed = IntPtr.Zero
         Me.LengthInternal = -1
         Me.ResetGcHandler()
         Me.Key = IntPtr.Zero
         Me.KeyLen = IntPtr.Zero
         Me.SecurePtr = IntPtr.Zero
         If _withInitNew Then
            Me.InitNew()
         End If
         GC.Collect()
         GC.WaitForPendingFinalizers()
      End Sub
#End Region
#Region "Methodes"
      <DebuggerNonUserCode>
      Private Sub WarmUp()
         Dim cnt = RSO.Next(0, 80)
         Enumerable.Range(0, cnt).ToList.
            ForEach(Sub(x) RSO.Next())
      End Sub
      <DebuggerNonUserCode>
      Private Sub InitNew()
         Me.IsBusy = True
         Me.InitializeSeed()
      End Sub
      <DebuggerNonUserCode>
      Private Sub InitializeSeed()
         Me.Gch = New List(Of GCHandle)
         Me.Seed = Me.ToPtr(Me.BitCrypt(Me.ToB64(SecureObject.SerializeBinary(RSO.Next)), True))
      End Sub
      <DebuggerNonUserCode>
      Private Sub ResetGcHandler()
         If Me.Gch Is Nothing Then Return
         For i As Int32 = Me.Gch.Count - 1 To 0 Step -1
            Me.Gch(i).Free()
         Next
         Me.Gch.Clear()
         Me.Gch = Nothing
      End Sub
      Private Sub SetKey(size As Int32)
         If size < 1 Then Return
         If Me.Gch Is Nothing Then Return
         If Not Me.KeyLen = IntPtr.Zero Then Return
         Me.RandomKey(size, RSO.Next(3, 6))
      End Sub
      Private Function GetKey() As Byte()()
         If Me.KeyLen = IntPtr.Zero Then Return Nothing
         Dim klen = SecureObject.DeserializeBinary(Of Int32())(Me.FromB64(Me.BitCrypt(Me.Decrypt(Me.ToBytes(Me.KeyLen, 48)), False)))
         Dim t = Me.BitCrypt(Me.Decrypt(Me.ToBytes(Me.Key, klen(1))), False)
         Dim br = BitConverter.GetBytes(klen(0))
         Me.ZeroFree(klen)
         Return SecureObject.DeserializeBinary(Of Byte()())(Me.FromB64(Me.XorCrypt(Me.FromB64(t), br)))
      End Function
      Private Function GetPlain() As Byte()
         If Me.IsDisposed Then Return Nothing
         Dim _key()() As Byte = Nothing
         Try
            _key = Me.GetKey()
            Dim bytes = Me.Decrypt(Me.ToBytes(Me.SecurePtr, Me.LengthInternal))
            For i As Int32 = _key.Length - 1 To 1 Step -1
               bytes = Me.XorCrypt(bytes, _key(i))
            Next
            Return Me.XorCrypt(Me.FromB64(bytes), _key(0))
         Finally
            If _key IsNot Nothing Then
               Me.ZeroFree(_key)
            End If
         End Try
      End Function
      Private Sub RandomKey(size As Int32, ksize As Int32)
         If size < 1 OrElse ksize < 1 Then Return
         Dim klen = Enumerable.Range(0, ksize).Select(Function(x) _
                     If(size < 30, size, RSO.Next(30, size))).ToArray
         Dim _key = klen.Select(Function(x) SecureObject.RandomKey(x)).ToArray
         Me.ZeroFree(klen)

         Dim t1 = Me.ToB64(SecureObject.SerializeBinary(_key))
         Me.ZeroFree(_key)

         Dim kklen = New Int32() {RSO.Next, 0}
         Dim br = BitConverter.GetBytes(kklen(0))
         Dim t2 = Me.BitCrypt(Me.ToB64(Me.XorCrypt(t1, br)), True)

         kklen(1) = t2.Length
         Me.Key = Me.ToPtr(Me.Encrypt(t2))
         Me.KeyLen = Me.ToPtr(Me.Encrypt(Me.BitCrypt(Me.ToB64(SecureObject.SerializeBinary(kklen)), True)))
         Me.ZeroFree(kklen)
      End Sub
      <DebuggerNonUserCode>
      Private Sub Swap(Of V As Structure)(ByRef _min As V, ByRef _max As V)
         'Swaps two Variable of T
         Dim tmp As V : tmp = _min
         _min = _max : _max = tmp
      End Sub
      Private Sub InitializeSO(ByRef plain() As Byte,
                  Optional _nothingplain As Boolean = True)
         If plain IsNot Nothing AndAlso plain.Length > 0 Then
            Me.SetKey(plain.Length)
            Dim _key = Me.GetKey()
            Dim bytes = Me.ToB64(Me.XorCrypt(plain, _key(0)))

            For i As Int32 = 1 To _key.Length - 1
               bytes = Me.XorCrypt(bytes, _key(i))
            Next
            Me.LengthInternal = bytes.Length
            Me.ZeroFree(_key)
            Me.SecurePtr = Me.ToPtr(Me.Encrypt(bytes))
            Return
         End If
         If _nothingplain Then Me.ZeroFree(plain)
      End Sub
      Private Function ToB64(ByRef plain() As Byte,
              Optional _setnothing As Boolean = True) As Byte()
         Try
            Return Encoding.UTF8.GetBytes(Convert.ToBase64String(plain))
         Finally
            If _setnothing Then Me.ZeroFree(plain)
         End Try
      End Function
      Private Function FromB64(ByRef b64() As Byte,
              Optional _setnothing As Boolean = True) As Byte()
         Try
            Return Convert.FromBase64String(Encoding.UTF8.GetString(b64))
         Finally
            If _setnothing Then Me.ZeroFree(b64)
         End Try
      End Function
      Private Function ToPtr(ByRef bytes() As Byte,
              Optional _setnothing As Boolean = True) As IntPtr
         If bytes Is Nothing OrElse bytes.Length < 1 Then Return IntPtr.Zero
         Me.Gch.Add(GCHandle.Alloc(bytes.ToArray, GCHandleType.Pinned))
         If _setnothing Then Me.ZeroFree(bytes)
         Return Me.Gch.Last.AddrOfPinnedObject
      End Function
      Private Sub ZeroFree(Of V As Structure)(ByRef arr()() As V,
                  Optional _setnothing As Boolean = True)
         Dim _arr = arr
         Enumerable.Range(0, arr.Length).ToList.ForEach(Sub(x) Me.ZeroFree(_arr(x)))
         If _setnothing Then arr = Nothing
      End Sub
      Private Sub ZeroFree(Of V As Structure)(ByRef arr() As V,
                  Optional _setnothing As Boolean = True)
         If arr Is Nothing OrElse arr.Length < 1 Then Return
         Dim sz = Marshal.SizeOf(GetType(V))
         Buffer.BlockCopy(New V(arr.Length - 1) {}, 0 * sz, arr, 0 * sz, arr.Length * sz)
         If _setnothing Then arr = Nothing
      End Sub
      Private Function Encrypt(ByRef plain() As Byte,
              Optional _nothingplain As Boolean = True) As Byte()
         If plain IsNot Nothing AndAlso plain.Length > 0 Then
            Dim res = plain.ToArray
            Dim rng = New Random(SecureObject.DeserializeBinary(Of Int32) _
                                (Me.FromB64(Me.BitCrypt(Me.ToBytes(Me.Seed, 72), False))))
            For i As Int32 = 0 To res.Length - 1
               Me.Swap(res(i), res(rng.Next Mod res.Length))
            Next
            If _nothingplain Then Me.ZeroFree(plain)
            Return res
         End If
         If _nothingplain Then Me.ZeroFree(plain)
         Return Nothing
      End Function
      Private Function Decrypt(ByRef cipher() As Byte,
              Optional _nothingcipher As Boolean = True) As Byte()
         If cipher IsNot Nothing AndAlso cipher.Length > 0 Then
            Dim res = cipher.ToArray
            Dim rng = New Random(SecureObject.DeserializeBinary(Of Int32) _
                                (Me.FromB64(Me.BitCrypt(Me.ToBytes(Me.Seed, 72), False))))
            Dim scale = Enumerable.Range(0, res.Length).
                        Select(Function(x) rng.Next).ToArray
            For i As Int32 = res.Length - 1 To 0 Step -1
               Me.Swap(res(i), res(scale(i) Mod res.Length))
            Next
            If _nothingcipher Then Me.ZeroFree(cipher)
            Return res
         End If
         If _nothingcipher Then Me.ZeroFree(cipher)
         Return Nothing
      End Function
      Private Function BitCrypt(ByRef bytes() As Byte, _crypt As Boolean,
                           Optional _setnothing As Boolean = True) As Byte()
         If bytes IsNot Nothing AndAlso bytes.Length > 0 Then
            Dim res = bytes.ToArray
            For i As Int32 = 0 To res.Length - 1
               If _crypt Then
                  res(i) = Me.BitCrypt(Convert.ToByte((res(i) + i) Mod 256))
               Else : res(i) = Convert.ToByte(Math.Abs(Convert.ToInt32(Me.BitCrypt(res(i))) + (((i \ 256) + 1) * 256) - i) Mod 256)
               End If
            Next
            If _setnothing Then Me.ZeroFree(bytes)
            Return res
         End If
         If _setnothing Then Me.ZeroFree(bytes)
         Return Nothing
      End Function
      Private Function BitCrypt(byt As Byte) As Byte
         Dim x = Convert.ToByte(((byt >> 1) Xor (byt >> 5)) And ((1 << 3) - 1))
         Return byt Xor ((x << 1) Or (x << 5))
      End Function
      Private Function ToBytes(ptr As IntPtr, size As Int32) As Byte()
         If ptr = IntPtr.Zero Then Return Nothing
         Dim res = New Byte(size - 1) {}
         Marshal.Copy(ptr, res, 0, res.Length)
         Return res
      End Function
      Private Function XorCrypt(ByRef bytes() As Byte, ByRef key() As Byte,
              Optional _setnothing As Boolean = True) As Byte()
         If key Is Nothing OrElse key.Length < 1 Then Return Nothing
         If bytes Is Nothing OrElse bytes.Length < 1 Then Return Nothing
         Try
            Dim k = key
            Return bytes.Select(Function(b, i) b Xor k(i Mod k.Length)).ToArray
         Finally
            If _setnothing Then Me.ZeroFree(key)
            If _setnothing Then Me.ZeroFree(bytes)
         End Try
      End Function
#End Region
#Region "Contructor - IDisposable"
      ''' <summary>
      ''' Contructor of SecureObject
      ''' </summary>
      <DebuggerNonUserCode>
      Public Sub New()
         Me.IsDisposed = False
         Me.Clear(False)
      End Sub
      <DebuggerNonUserCode>
      Protected Overrides Sub Finalize()
         Me.Dispose(True)
         MyBase.Finalize()
      End Sub
      ''' <summary>
      ''' Resets all resources
      ''' </summary>
      <DebuggerNonUserCode>
      Public Sub Dispose() Implements IDisposable.Dispose
         Me.Dispose(True)
         GC.SuppressFinalize(Me)
      End Sub
      <DebuggerNonUserCode>
      Private Sub Dispose(disposing As Boolean)
         If Not Me.IsDisposed Then
            If disposing Then
               'WICHTIG
               'Die IntPtr-Variablen müssen nicht zurückgesetzt 
               'werden, da sie in den verwalteten Speicherbereich 
               'zeigen.
            End If
            Me.Clear(False)
         End If
         Me.IsBusy = True
         Me.IsDisposed = True
      End Sub
#End Region
   End Class

   ''' <summary>
   ''' Provides various methods
   ''' </summary>
   <DebuggerNonUserCode>
   Partial Public NotInheritable Class SecureObject
#Region "Shared/Static Entries"
      ''' <summary>
      ''' Create a new Random Array of Char 
      ''' </summary>
      ''' <param name="size">Size</param>
      ''' <returns>Return a Array of Char</returns>
      Public Shared Function GetRngChar(size As Int32) As Char()
         If size < 1 Then Return Nothing
         Dim _min = 32I, _max = Convert.ToInt32(Char.MaxValue)
         Dim tmp = New Char(size - 1) {}
         For i As Int32 = 0 To size - 1
            Dim ch = Convert.ToChar(RSO.Next(_min, _max))
            If Char.IsLetterOrDigit(ch) OrElse
            Char.IsWhiteSpace(ch) Then
               tmp(i) = ch
            Else : i -= 1
            End If
         Next
         Return tmp
      End Function
      ''' <summary>
      ''' Return random alphabetic
      ''' </summary>
      ''' <param name="size">Array-Size</param>
      ''' <returns>Return a random Array with digits</returns>
      Public Shared Function GetRndAlpha(ByVal size As Int32) As Char()
         If size > 0 Then
            Dim res As New StringBuilder(size)
            Dim alpha As String = AlphaString() _
             .OrderBy(Function(s) RSO.Next).ToArray
            For i As Int32 = 0 To size - 1
               res.Append(alpha(RSO.Next(0, alpha.Length - 1)))
            Next
            Return res.ToString.ToCharArray
         End If
         Return Nothing
      End Function
      ''' <summary>
      ''' Return a Array of Bytes with Char-Size fixed in UTF8
      ''' </summary>
      ''' <param name="size">Size for Array of Char</param>
      ''' <returns>Return e Array of Bytes (Char-Convert)</returns>
      Public Shared Function GetRngCharBytesU8(size As Int32) As Byte()
         Return Encoding.UTF8.GetBytes(GetRngChar(size))
      End Function
      ''' <summary>
      ''' Return a Array of Bytes with Bytes-Size fixed in UTF8
      ''' </summary>
      ''' <param name="size">Size for Array of Bytes</param>
      ''' <returns>Return e Array of Bytes (Char-Convert)</returns>
      Public Shared Function GetRngCharBytesU8Fix(size As Int32) As Byte()
         'Es wird genau eine Anzahl Bytes zur Verfügung gestellt.
         If size <= 3 Then Throw New ArgumentException("size > 3")
         Dim btmp(size) As Byte 'So ist korrekt
         Dim tmp() As Char = Nothing
         While btmp.Length > size - 1
            tmp = GetRngChar(size \ 3)
            btmp = Encoding.UTF8.GetBytes(tmp)
         End While
         Dim alpha = GetRndAlpha(size - btmp.Length)
         Dim chars = New Char(tmp.Length + alpha.Length - 1) {}
         Array.Copy(tmp, chars, tmp.Length)
         Array.Copy(alpha, 0, chars, tmp.Length, alpha.Length)
         Shuffel(chars)
         Dim res = Encoding.UTF8.GetBytes(chars)
         If res.Length = size Then
            Return res
         End If
         Return Nothing
      End Function
      Public Shared Function SerializeBinary(Of V)(obj As V) As Byte()
         If obj Is Nothing Then Return Nothing
         Dim res() As Byte
         Using ms = New MemoryStream
            Dim binformatter As New BinaryFormatter()
            binformatter.Serialize(ms, obj)
            res = ms.ToArray
         End Using
         Return res
      End Function
      Public Shared Function DeserializeBinary(Of V)(ByRef data() As Byte,
                             Optional _setnothing As Boolean = True) As V
         If data Is Nothing OrElse data.Length < 1 Then Return Nothing
         Using ms = New MemoryStream(data)
            Try
               Dim binformatter As New BinaryFormatter()
               Return DirectCast(binformatter.Deserialize(ms), V)
            Catch ex As Exception
               Return Nothing
            Finally
               If _setnothing Then
                  Dim b = New Byte(data.Length - 1) {}
                  Buffer.BlockCopy(b, 0, data, 0, b.Length)
                  b = Nothing : data = Nothing
               End If
            End Try
         End Using
         Return Nothing
      End Function
      Public Shared Function RandomKey(size As Int32) As Byte()
         Dim t As Byte
         Dim res = New Byte(size - 1) {}
         For i As Int32 = 0 To size - 1
            t = Convert.ToByte(RSO.Next Mod 256)
            If t = 0 Then i -= 1 : Continue For
            res(i) = t
         Next
         Return res
      End Function
#End Region
#Region "Shared/Static Methodes"
      ''' <summary>
      ''' Return the alphabetic
      ''' </summary>
      ''' <returns>Return alphabetic digits</returns>
      Private Shared Function AlphaString() As String
         Dim res As New StringBuilder(62)
         Dim z() As Byte = {48, 57, 65, 90, 97, 122}
         For i As Int32 = 0 To z.Length - 1 Step 2
            For j As Byte = z(i) To z(i + 1)
               res.Append(Convert.ToChar(j))
            Next
         Next
         Return res.ToString
      End Function
      Private Shared Sub Shuffel(Of T)(ByRef arr() As T)
         If arr Is Nothing Then Return
         Dim tmp = Enumerable.Range(0, arr.Length).
                OrderBy(Function(x) RSO.Next).ToArray
         Dim c As T
         For i As Int32 = 0 To arr.Length - 1
            c = arr(i)
            arr(i) = arr(tmp(i))
            arr(tmp(i)) = c
         Next
      End Sub
#End Region
#Region "Private Constructor"
      Private Sub New()
      End Sub
#End Region
   End Class

   Namespace MyRngSecureObject
#Region "SOCryptoRandom"
      ''' <summary>
      ''' Simple Int32 Random
      ''' </summary>
      <DebuggerNonUserCode>
      Friend NotInheritable Class SOCR
#Region "Shared/Static Members/Variables"
         ''' <summary>
         ''' RNGCryptoServiceProvider Variable
         ''' </summary>
         <Browsable(False)>
         <DebuggerBrowsable(DebuggerBrowsableState.Never)>
         <DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)>
         Private Shared Rand As New RNGCryptoServiceProvider
#End Region
#Region "Shared/Static Crypto-Random"
         ''' <summary>
         ''' Next Random Int32 (without Exceptions)
         ''' </summary>
         ''' <returns>Return a Random Int32</returns>
         Friend Shared Function [Next]() As Int32
            'Crypto-random number of Int32
            Dim scale = -1I
            Dim b() As Byte = Nothing
            Dim sz = Marshal.SizeOf(GetType(Int32))
            While scale < 0I
               b = New Byte(sz - 1) {}
               Rand.GetNonZeroBytes(b)
               scale = Convert.ToInt32(b(0)) Or (Convert.ToInt32(b(1)) << 8I) Or
                       (Convert.ToInt32(b(2)) << 16I) Or (Convert.ToInt32(b(3)) << 24I)
            End While
            Return Math.Abs(scale)
         End Function
         ''' <summary>
         ''' Random Int32 in Range min - max (without Exceptions)
         ''' </summary>
         ''' <param name="_min">Least legal value for the Random number.</param>
         ''' <param name="_max">One greater than the greatest legal return value.</param>
         ''' <returns>Return a Int32 [min - max]</returns>
         Friend Shared Function [Next](_min As Int32, _max As Int32) As Int32
            'Crypto-random number of Int32 with min - max
            If _min = _max Then Return _min
            SwapCheck(_min, _max)
            Dim scale = SOCR.[Next]()
            Return Math.Abs(Convert.ToInt32(_min + (_max - _min) * (scale / CDbl(Int32.MaxValue))))
         End Function
         ''' <summary>
         ''' Swaps two Int32 if necessary
         ''' </summary>
         ''' <typeparam name="T">Generic Datatyp</typeparam>
         ''' <param name="_min">Least legal value.</param>
         ''' <param name="_max">Greater legal value.</param>
         Private Shared Sub SwapCheck(Of T As IComparable(Of T))(ByRef _min As T, ByRef _max As T)
            'Swaps two Variable of T 
            If _min.CompareTo(_max) > 0 Then
               Dim tmp As T : tmp = _min
               _min = _max : _max = tmp
            End If
         End Sub
         Private Sub New()
         End Sub
#End Region
      End Class
#End Region
   End Namespace
End Namespace